﻿/*
* kner 2015
* key debounce version 3 (usable)
* problem: the buttons bounce as well on pressing as on releasing the button
*
* Good: no blocking of the ISR
* Caveat: short spikes/pressures are not executed
* connect a button on Arduino Pin 2 to GND
*/

#include <Arduino.h>

#define DEBUG_LCD
#include "_debug.h"
int timerOverflows; 
int numberofKeyPressures;
int pendingInterrupsCounter;
int nrofInts;

void setup()
{   //millis() not needed
	cli(); 
	TIMSK0 &=~_BV(TOIE0);
	
	EICRA |= _BV(ISC01); //Falling edge
	EIMSK |= _BV(INT0); //enable external Interrupt 0

	//Timer2 Presc 1024 => Overflow period 16ms
	TCCR2A &=~(_BV(WGM21)|_BV(WGM20)); //normal Mode
	TIMSK2 |= _BV(TOIE2); //overflow enable
	
	pinMode(2,INPUT_PULLUP); //ext. Interrupt
	sei(); 
}

void loop()
{   volatile static int dummy=0; dummy++;  //needed for simulator if no additional lines of code
}


ISR(INT0_vect){ //multiple executions per button push and button release

  TCCR2B |= _BV(CS22)|_BV(CS21)|_BV(CS20); //start debounce timer
  TCNT2 = 0; //count=0
  R(++nrofInts)
 
}

ISR(TIMER2_OVF_vect){  //every 16ms   ==> count overflows for longer timings
	//Debounce Time finished
	TCCR2B &= ~0b00000111; //turn off timer2 
	if (digitalRead(2)==LOW){  //prevent bouncing on release of the button
		 L(++numberofKeyPressures)
		 //start code for key recognized
		 //...
	}
}
